use HTTP::Status qw(:constants);
use List::Util qw(max);
use Raisin::API;
use Types::Standard qw(HashRef Any Int Str);
​
api_format 'json';
plugin 'Logger', fallback => 1;
plugin 'Swagger';
​
my %USERS = (
    1 => {
        name => 'Sample Taro',
        email => 'taro@sample.com',
    },
    2 => {
        name => 'Sample Jiro',
        email => 'jiro@sample.com',
    },
);
​
desc 'Users API';
resource users => sub {
    summary 'List all users at once';
    get 'all' => sub {
        my @users
            = map { { id => $_, %{ $USERS{$_} } } }
              sort { $a <=> $b } keys %USERS;
        { data => \@users }
    };
​
    desc 'Actions on the user';
    params requires('id', type => Int, desc => 'User ID');
    route_param 'id' => sub {
        summary 'Show user';
        get sub {
            my $params = shift;
            $USERS{ $params->{id} };
        };
​
        summary 'Delete user';
        del sub {
            my $params = shift;
            delete $USERS{ $params->{id} };
            res->status(HTTP_NO_CONTENT);
            undef;
        };
    };
​
    summary 'Create new user';
    params(
        requires('user', type => HashRef, desc => 'User object', group {
            requires('name', type => Str, desc => 'User name'),
            optional('email', type => Str, default => undef, regex => qr/.+\@.+/, desc => 'User email'),
        }),
    );
    post sub {
        my $params = shift;
​
        my $id = max(keys %USERS) + 1;
        $USERS{$id} = $params->{user};
​
        res->status(HTTP_CREATED);
        { success => 1 }
    };
};
​
before sub {
  my $self = shift;
  app->log( debug => $self->req->method . "\t" . $self->req->path );
};
​
after sub {
    app->log( debug => "Success" );
};

run;